@@ -284,8 +284,6 @@ const char* const SPVM_OPCODE_C_ID_NAMES[] = {
   "NEW_FLOAT_ARRAY",
   "NEW_DOUBLE_ARRAY",
   "CONCAT",
-  "PUSH_EVAL",
-  "POP_EVAL",
   "GET_PACKAGE_VAR_BYTE",
   "GET_PACKAGE_VAR_SHORT",
   "GET_PACKAGE_VAR_INT",
