@@ -284,8 +284,6 @@ enum {
   SPVM_OPCODE_C_ID_NEW_FLOAT_ARRAY,
   SPVM_OPCODE_C_ID_NEW_DOUBLE_ARRAY,
   SPVM_OPCODE_C_ID_CONCAT,
-  SPVM_OPCODE_C_ID_PUSH_EVAL,
-  SPVM_OPCODE_C_ID_POP_EVAL,
   SPVM_OPCODE_C_ID_GET_PACKAGE_VAR_BYTE,
   SPVM_OPCODE_C_ID_GET_PACKAGE_VAR_SHORT,
   SPVM_OPCODE_C_ID_GET_PACKAGE_VAR_INT,
