@@ -19,8 +19,13 @@ struct SPVM_package {
   SPVM_HASH* op_package_var_symtable;
   SPVM_LIST* op_subs;
   SPVM_HASH* op_sub_symtable;
-  SPVM_LIST* object_field_rel_ids;
-  SPVM_HASH* method_signature_symtable;
+  SPVM_LIST* object_field_indexes;
+  SPVM_LIST* sub_signatures;
+  SPVM_HASH* sub_signature_symtable;
+  SPVM_LIST* field_signatures;
+  SPVM_HASH* field_signature_symtable;
+  SPVM_LIST* package_var_signatures;
+  SPVM_HASH* package_var_signature_symtable;
   SPVM_HASH* has_interface_cache_symtable;
   SPVM_OP* op_sub_destructor;
   const char* load_path;
@@ -29,12 +34,6 @@ struct SPVM_package {
   _Bool is_private;
   _Bool is_anon;
   int32_t category;
-  SPVM_LIST* op_package_var_accesses;
-  SPVM_LIST* op_field_accesses;
-  SPVM_LIST* op_types;
-  SPVM_LIST* op_call_subs;
-  SPVM_LIST* op_constants;
-  SPVM_LIST* op_switch_infos;
 };
 
 SPVM_PACKAGE* SPVM_PACKAGE_new(SPVM_COMPILER* compiler);
